#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author Komal Thareja (kthare10@renci.org)
import unittest

from fss_utils.jwt_manager import JWTManager, ValidateCode


class JWTManagerTest(unittest.TestCase):
    def test_encode_and_sign_with_private_key_file_does_not_exist(self):
        claims = {"foo": "bar"}
        code, exception = JWTManager.encode_and_sign_with_private_key(validity=90, claims=claims,
                                                                      private_key_file_name="abc.pem",
                                                                      kid="abcd-kid", algorithm="RS256")
        self.assertEqual(code, ValidateCode.UNABLE_TO_LOAD_KEYS)
        self.assertIsNotNone(exception)
        self.assertTrue(isinstance(exception, Exception))
        self.assertFalse(isinstance(exception, str))

    def test_encode_and_sign_with_valid_key_no_passphrase(self):
        claims = {"foo": "bar"}
        code, token = JWTManager.encode_and_sign_with_private_key(validity=90, claims=claims,
                                                                  private_key_file_name="test/data/privkey.pem",
                                                                  kid="abcd-kid", algorithm="RS256")
        self.assertEqual(code, ValidateCode.VALID)
        self.assertIsNotNone(token)
        self.assertFalse(isinstance(token, Exception))
        self.assertTrue(isinstance(token, str))

        code, decoded_token = JWTManager.decode(cookie=token, verify=False, compression=False)

        self.assertEqual(code, ValidateCode.VALID)
        for key, value in claims.items():
            self.assertIsNotNone(decoded_token.get(key), value)

    def test_encode_jwk_with_non_existent_pem(self):
        code, exception = JWTManager.encode_jwk(key_file_name="abc.pem", kid="kid", alg="RS256")
        self.assertEqual(code, ValidateCode.UNABLE_TO_LOAD_KEYS)
        self.assertIsNotNone(exception)
        self.assertTrue(isinstance(exception, Exception))
        self.assertFalse(isinstance(exception, str))

    def test_encode_jwk_with_valid_pem(self):
        code, token = JWTManager.encode_jwk(key_file_name="test/data/pubkey.pem", kid="kid", alg="RS256")
        self.assertEqual(code, ValidateCode.VALID)
        self.assertIsNotNone(token)
        self.assertFalse(isinstance(token, Exception))
        self.assertTrue(isinstance(token, dict))

    def test_encode_and_compress_enabled(self):
        claims = {"foo": "bar"}
        code, token = JWTManager.encode_and_compress(claims=claims, secret="super-secret", validity=500)
        self.assertEqual(code, ValidateCode.VALID)

        code, decoded_token = JWTManager.decode(cookie=token, verify=False, compression=True)

        self.assertEqual(code, ValidateCode.VALID)
        for key, value in claims.items():
            self.assertIsNotNone(decoded_token.get(key), value)

    def test_encode_and_compress_disabled(self):
        claims = {"foo": "bar"}
        code, token = JWTManager.encode_and_compress(claims=claims, secret="super-secret", validity=500,
                                                     compression=False)
        self.assertEqual(code, ValidateCode.VALID)

        code, decoded_token = JWTManager.decode(cookie=token, verify=False, compression=False)

        self.assertEqual(code, ValidateCode.VALID)
        for key, value in claims.items():
            self.assertIsNotNone(decoded_token.get(key), value)



import unittest
import jwt
import time
import datetime

from fss_utils.jwt_validate import JWTValidator
from fss_utils.jwt_manager import ValidateCode


class JWTTester(unittest.TestCase):

    def setUp(self):
        self.url = "https://cilogon.org/oauth2/certs"
        self.right_audience = "cilogon:/client_id/73bb503829ff96ad948f41b1d8e2a2a5"
        self.wrong_audience = "cilogon:/client_id/1234567890"
        self.period = datetime.timedelta(seconds=10)
        self.default_validator = JWTValidator(url=self.url, refresh_period=self.period)
        self.validator = None
        self.testToken = {"email": "user@domain.net", "given_name": "Some", "family_name": "One",
                          "name": "Some One", "iss": "https://cilogon.org", "aud": "cilogon:foo",
                          "sub": "https://cilogon.org/serverT/users/241998",
                          "token_id": "https://cilogon.org/oauth2/idToken/1234567898",
                          "auth_time": "1607382404", "iat": 1607382405, "roles": [
                            "CO:members:all",
                            "CO:members:active",
                            "CO:admins",
                            "CO:COU:project-leads:members:active",
                            "CO:COU:project-leads:members:all",
                            "CO:COU:abf0014e-72f5-44ab-ac63-5ec5a5debbb8-pm:members:active",
                            "CO:COU:abf0014e-72f5-44ab-ac63-5ec5a5debbb8-pm:members:all"
                            ],
                          'exp': int(time.time()) + 1000}
        self.testToken2 = "eyJ0eXAiOiJKV1QiLCJraWQiOiIyNDRCMjM1RjZCMjhFMzQxMDhEMTAxRUFDNzM2MkM0RSIsImFsZyI6IlJTMjU2I" \
                          "n0.eyJlbWFpbCI6ImliYWxkaW5AcmVuY2kub3JnIiwiZ2l2ZW5fbmFtZSI6IklseWEiLCJmYW1pbHlfbmFtZSI6Ik" \
                          "JhbGRpbiIsIm5hbWUiOiJJbHlhIEJhbGRpbiIsImlzcyI6Imh0dHBzOi8vY2lsb2dvbi5vcmciLCJzdWIiOiJodHR" \
                          "wOi8vY2lsb2dvbi5vcmcvc2VydmVyVC91c2Vycy8yNDE5OTgiLCJhdWQiOiJjaWxvZ29uOi9jbGllbnRfaWQvNzNi" \
                          "YjUwMzgyOWZmOTZhZDk0OGY0MWIxZDhlMmEyYTUiLCJ0b2tlbl9pZCI6Imh0dHBzOi8vY2lsb2dvbi5vcmcvb2F1d" \
                          "GgyL2lkVG9rZW4vMTk0ZGFiMDUzMzcxMGY0ZjM0YTEzNjYxNzQyMDQ1NTIvMTYwNzk3ODE1NDk5MyIsImF1dGhfdG" \
                          "ltZSI6IjE2MDc5NzgxNTMiLCJleHAiOjE2MDc5NzkwNTQsImlhdCI6MTYwNzk3ODE1NCwicm9sZXMiOlsiQ086bWV" \
                          "tYmVyczphbGwiLCJDTzptZW1iZXJzOmFjdGl2ZSIsIkNPOmFkbWlucyIsIkNPOkNPVTpwcm9qZWN0LWxlYWRzOm1l" \
                          "bWJlcnM6YWN0aXZlIiwiQ086Q09VOnByb2plY3QtbGVhZHM6bWVtYmVyczphbGwiLCJDTzpDT1U6YWJmMDAxNGUtN" \
                          "zJmNS00NGFiLWFjNjMtNWVjNWE1ZGViYmI4LXBtOm1lbWJlcnM6YWN0aXZlIiwiQ086Q09VOmFiZjAwMTRlLTcyZj" \
                          "UtNDRhYi1hYzYzLTVlYzVhNWRlYmJiOC1wbTptZW1iZXJzOmFsbCJdfQ.Uats19baYgA8mOllbPIDN_cpbXnPLPHu" \
                          "_QE5cBjlgu4KH7DSv0_15d-wjY59gJQUPq42Dg4cbgEzsfrNh_I7GldYCbopCWSv5S7rudJKbiz-gInPmDITGuOFH" \
                          "luOXYkEzJNJ1uxlUOvvQtyrvsCM2DuvSrd2FeBIY0MRHI92UEuvpjiCKn5JD-PhcFv8CUixityBXwewICvFo9k7YV" \
                          "70PvaL0SFzKskaofrdL8HialvWgWO26qEXsF2_CSUkgmTI_GikuVamUU3eP8jP7TntsqD_TBXKIKis2y8Kg9ao5N9" \
                          "OPu8sZ3Y9DTtqLeJKp7tsLsdGe6F_m58ozRck61_trA"

        self.testToken3 = "eyJ0eXAiOiJKV1QiLCJraWQiOiIyNDRCMjM1RjZCMjhFMzQxMDhEMTAxRUFDNzM2MkM0RSIsImFsZyI6IlJTMjU2I" \
                          "n0.eyJlbWFpbCI6ImliYWxkaW5AcmVuY2kub3JnIiwiZ2l2ZW5fbmFtZSI6IklseWEiLCJmYW1pbHlfbmFtZSI6Ik" \
                          "JhbGRpbiIsIm5hbWUiOiJJbHlhIEJhbGRpbiIsImlzcyI6Imh0dHBzOi8vY2lsb2dvbi5vcmciLCJzdWIiOiJodHR" \
                          "wOi8vY2lsb2dvbi5vcmcvc2VydmVyVC91c2Vycy8yNDE5OTgiLCJhdWQiOiJjaWxvZ29uOi9jbGllbnRfaWQvNzNi" \
                          "YjUwMzgyOWZmOTZhZDk0OGY0MWIxZDhlMmEyYTUiLCJ0b2tlbl9pZCI6Imh0dHBzOi8vY2lsb2dvbi5vcmcvb2F1d" \
                          "GgyL2lkVG9rZW4vMTk0ZGFiMDUzMzcxMGY0ZjM0YTEzNjYxNzQyMDQ1NTIvMTYwNzk3ODE1NDk5MyIsImF1dGhfdG" \
                          "ltZSI6IjE2MDc5NzgxNTMiLCJleHAiOjE2MDc5NzkwNTQsImlhdCI6MTYwNzk3ODE1NCwicm9sZXMiOlsiQ086bWV" \
                          "tYmVyczphbGwiLCJDTzptZW1iZXJzOmFjkNPOmFkbWlucyIsIkNPOkNPVTpwcm9qZWN0LWxlYWRzOm1l" \
                          "bWJlcnM6YWN0aXZlIiwiQ086Q09VOnByb2plY3QtbGVhZHM6bWVtYmVyczphbGwiLCJDTzpDT1U6YWJmMDAxNGUtN" \
                          "zJmNS00NGFiLWFjNjMtNWVjNWE1ZGViYmI4LXBtOm1lbWJlcnM6YWN0aXZlIiwiQ086Q09VOmFiZjAwMTRlLTcyZj" \
                          "UtNDRhYi1hYzYzLTVlYzVhNWRlYmJiOC1wbTptZW1iZXJzOmFsbCJdfQ.Uats19baYgA8mOllbPIDN_cpbXnPLPHu" \
                          "_QE5cBjlgu4KH7DSv0_15d-wjY59gJQUPq42Dg4cbgEzsfrNh_I7GldYCbopCWSv5S7rudJKbiz-gInPmDITGuOFH" \
                          "luOXYkEzJNJ1uxlUOvvQtyrvsCM2DuvSrd2FeBIY0MRHI92UEuvpjiCKn5JD-PhcFv8CUixityBXwewICvFo9k7YV" \
                          "70PvaL0SFzKskaofrdL8HialvWgWO26qEXsF2_CSUkgmTI_GikuVamUU3eP8jP7TntsqD_TBXKIKis2y8Kg9ao5N9" \
                          "OPu8sZ3Y9DTtqLeJKp7tsLsdGe6F_m58ozRck61_trA"

    def testFetchKeys(self):
        """ test fetching keys from a real endpoint (succeed) """
        self.validator = JWTValidator(url=self.url, refresh_period=self.period)
        vc, e = self.validator.fetch_pub_keys()
        assert vc is None and e is None

    def testEncodeDecode(self):
        """ test simple symmetric encoding/decoding (succeed) """
        encoded_token = jwt.encode(self.testToken, key='secret', algorithm='HS256')
        jwt.decode(encoded_token, key='secret', algorithms=['HS256'],
                   options={"verify_exp": True, "verify_aud": True},
                   audience='cilogon:foo')

    def testNoExpiredNoAudience(self):
        """ validate signature, ignore expiration and audience (succeed) """
        # audience is set to None in this case
        vc, e = self.default_validator.validate_jwt(token=self.testToken2)
        assert vc is ValidateCode.VALID

    def testNoExpiredRightAudience(self):
        """ validate signature, ignore expiration, check audience (succeed) """
        self.validator = JWTValidator(url=self.url, refresh_period=self.period,
                                      audience=self.right_audience)
        vc, e = self.validator.validate_jwt(token=self.testToken2)
        assert vc is ValidateCode.VALID

    def testNoExpiredWrongAudience(self):
        """ validate signature, ignore expiration, check audience (fail) """
        self.validator = JWTValidator(url=self.url, refresh_period=self.period,
                                      audience=self.wrong_audience)
        vc, e = self.validator.validate_jwt(token=self.testToken2)
        assert vc is ValidateCode.INVALID and str(e) == "Invalid audience"

    def testStrictExpired(self):
        """ test for expiration (fail) """
        # audience is set to None in this case
        vc, e = self.default_validator.validate_jwt(token=self.testToken2, verify_exp=True)
        assert vc is ValidateCode.INVALID and str(e) == "Signature has expired"

    def testBadToken(self):
        """ test an unparsable token (fail) """
        vc, e = self.default_validator.validate_jwt(token=self.testToken3)
        assert vc is ValidateCode.UNPARSABLE_TOKEN

    def testKeyRefetch(self):
        """ test key refetch from JWK endpoint """
        vc, e = self.default_validator.validate_jwt(token=self.testToken2)
        assert vc is ValidateCode.VALID
        print("Sleeping for 5 seconds")
        time.sleep(5)
        vc, e = self.default_validator.validate_jwt(token=self.testToken2)
        assert vc is ValidateCode.VALID
        print("Sleeping for 6 seconds")
        time.sleep(6)
        vc, e = self.default_validator.validate_jwt(token=self.testToken2)
        assert vc is ValidateCode.VALID



import requests
import jwt
import json
import datetime

from .jwt_manager import ValidateCode


class JWTValidator:
    """This class caches keys retrieved from a specified endpoint
    and uses them to validate provided JWTs"""

    def __init__(self, *, url, refresh_period, audience=None):
        """ Initialize a validator with an endpoint URL presenting JWKs,
        a refresh period for keys expressed as datetime.timedelta and
        audience (i.e. CI Logon client id cilogon:/client_id/1234567890).
        :param url:
        :param refresh_period:
        :param audience:
        """
        self.url = url
        self.aud = audience
        assert refresh_period is None or isinstance(refresh_period, datetime.timedelta)
        self.cachePeriod = refresh_period
        self.pubKeys = None
        self.keysFetched = None

    def fetch_pub_keys(self):
        """
        Fetch JWKs from an endpoint, return a dictionary of key ids vs public key values (RSA)
        Returns a tuple ValidateCode, public key dict. Code is None in case of success,
        dict is none in case of failure.
        :return ValidateCode or None, exception or None:
        """
        if self.keysFetched is not None:
            if datetime.datetime.now() < self.keysFetched + self.cachePeriod:
                return None, None

        r = requests.get(self.url)
        if r.status_code != 200:
            return ValidateCode.UNABLE_TO_FETCH_KEYS, None

        self.keysFetched = datetime.datetime.now()

        try:
            self.pubKeys = dict()
            jwks = json.loads(r.text)
            for jwk in jwks['keys']:
                kid = jwk['kid']
                self.pubKeys[kid] = jwt.algorithms.RSAAlgorithm.from_jwk(json.dumps(jwk))
            return None, None
        except Exception as e:
            return ValidateCode.UNABLE_TO_DECODE_KEYS, e

    def validate_jwt(self, *, token, verify_exp=False):
        """
        Validate a token using a JWKs object retrieved from an endpoint.
        Returns a tuple ValidateCode code, exception object if it occurred (or None).
        Requires the token, the endpoint URL and the audience, i.e. CI Logon client ID
        cilogon:/client_id/1234567890
        You can turn on expiration validation (off by default), but beware most tokens
        live just a few minutes.
        :param token:
        :param verify_exp:
        :return tuple ValdateCode, Exception:
        """
        r, e = self.fetch_pub_keys()
        if r is not None:
            return r, e

        # get kid from token
        try:
            kid = jwt.get_unverified_header(token).get('kid', None)
            alg = jwt.get_unverified_header(token).get('alg', None)
        except jwt.DecodeError as e:
            return ValidateCode.UNPARSABLE_TOKEN, e

        if kid is None:
            return ValidateCode.UNSPECIFIED_KEY, None

        if alg is None:
            return ValidateCode.UNSPECIFIED_ALG, None

        if kid not in self.pubKeys.keys():
            return ValidateCode.UNKNOWN_KEY, None

        key = self.pubKeys[kid]

        options = dict()
        if verify_exp:
            options["verify_exp"] = True
        else:
            options["verify_exp"] = False

        if self.aud is None:
            options["verify_aud"] = False
        else:
            options["verify_aud"] = True

        # options https://pyjwt.readthedocs.io/en/latest/api.html
        try:
            decoded_token = jwt.decode(token, key=key, algorithms=[alg], options=options, audience=self.aud)
        except Exception as e:
            return ValidateCode.INVALID, e

        return ValidateCode.VALID, decoded_token



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author Komal Thareja (kthare10@renci.org)
from enum import Enum
from typing import Set, List, Tuple, Union

from .jwt_manager import JWTManager, ValidateCode


class CustomClaimsType(Enum):
    """
    Defines the Custom Claims Types
    """
    OPENID = 1
    EMAIL = 2
    PROFILE = 3
    CILOGON_USER_INFO = 4

    def interpret(self) -> Set[str]:
        interpretations = {
            1: {'sub', 'iss', 'aud', 'token_id'},
            2: {'email'},
            3: {'given_name', 'family_name', 'name'},
            4: {'idp', 'idp_name', 'eppn', 'eptid', 'affiliation', 'ou', 'oidc', 'cert_subject_dn'}
          }
        return interpretations[self.value]


class PTokens:
    """
    Encapsulates all the tokens to be included in the Cookie
    Assumes all the passed tokens are valid and decoded Identity claims are specified as well
    """
    def __init__(self, *, id_token: str, idp_claims: dict, access_token: str = None, refresh_token: str = None):
        """
        :param id_token identity token
        :param idp_claims identity claims
        :param access_token access token
        :param refresh_token refresh token
        """
        self.access_token = access_token
        self.refresh_token = refresh_token
        self.id_token = id_token
        self.idp_claims = idp_claims


class VouchEncoder:
    """
    Responsible for decoding/encoding a Vouch Cookie
    """
    def __init__(self, *, secret: str, compression: bool = True):
        """
        :param secret vouch jwt secret
        :param compression flag indicating if compression is enabled/disabled
        """
        self.secret = secret
        self.compression = compression

    @staticmethod
    def _generate_custom_claims(*, idp_claims: dict,
                                custom_claims_to_include: List[CustomClaimsType]) -> Tuple[dict, str]:
        """
        Generate the custom claims based on the IDP Claims as per the configuration
        :param idp_claims IDP Claims
        :param custom_claims_to_include custom claims to be included in the cookie

        :return Custom Claims or None, username or None
        """
        result = {}
        username = None
        for claim_type in custom_claims_to_include:
            for claim in claim_type.interpret():
                value = idp_claims.get(claim, None)
                if value is not None:
                    result[claim] = value
                if claim == "email":
                    username = value
        return result, username

    def encode(self, *, custom_claims_type: List[CustomClaimsType], p_tokens: PTokens,
               validity_in_seconds: int) -> Tuple[ValidateCode, Union[Exception, str]]:
        """
        Encodes a vouch cookie
        :param custom_claims_type list of custom claims to be included in the cookie
        :param p_tokens Provider Tokens and claims
        :param validity_in_seconds validity in seconds for the cookie

        :return ValidateCode or None, exception or None, vouch cookie or None
        """
        if p_tokens.id_token is None or p_tokens.idp_claims is None:
            return ValidateCode.UNSPECIFIED_ID_TOKEN, None

        custom_claims, username = self._generate_custom_claims(idp_claims=p_tokens.idp_claims,
                                                               custom_claims_to_include=custom_claims_type)

        vouch_claims = {"username": username, "sites": [], "CustomClaims": custom_claims, "PIdToken": p_tokens.id_token}

        if p_tokens.access_token is not None:
            vouch_claims["PAccessToken"] = p_tokens.access_token

        if p_tokens.refresh_token is not None:
            vouch_claims["PRefreshToken"] = p_tokens.refresh_token

        return JWTManager.encode_and_compress(claims=vouch_claims, secret=self.secret,
                                              compression=self.compression,
                                              validity=validity_in_seconds)



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author Komal Thareja (kthare10@renci.org)
import unittest
import time

from fss_utils.jwt_manager import ValidateCode, JWTManager
from fss_utils.vouch_encoder import VouchEncoder, CustomClaimsType, PTokens


class VouchEncoderTest(unittest.TestCase):
    def setUp(self):
        self.valid_token = "eyJ0eXAiOiJKV1QiLCJraWQiOiIyNDRCMjM1RjZCMjhFMzQxMDhEMTAxRUFDNzM2MkM0RSIsImFsZyI6IlJTMjU2I" \
                           "n0.eyJlbWFpbCI6Imt0aGFyZTEwQGVtYWlsLnVuYy5lZHUiLCJnaXZlbl9uYW1lIjoiS29tYWwiLCJmYW1pbHlfbm" \
                           "FtZSI6IlRoYXJlamEiLCJuYW1lIjoiS29tYWwgVGhhcmVqYSIsImlzcyI6Imh0dHBzOi8vY2lsb2dvbi5vcmciLCJ" \
                           "zdWIiOiJodHRwOi8vY2lsb2dvbi5vcmcvc2VydmVyQS91c2Vycy8xMTkwNDEwMSIsImF1ZCI6ImNpbG9nb246L2Ns" \
                           "aWVudF9pZC83M2JiNTAzODI5ZmY5NmFkOTQ4ZjQxYjFkOGUyYTJhNSIsInRva2VuX2lkIjoiaHR0cHM6Ly9jaWxvZ" \
                           "29uLm9yZy9vYXV0aDIvaWRUb2tlbi82ZGVjZDNmNDI2ODZmMDJiZDUzNTY0MDRlYTZmNTc4MS8xNjEwNDAwODE5NT" \
                           "U5IiwiYXV0aF90aW1lIjoiMTYxMDQwMDYzMyIsImV4cCI6MTYxMDQwMTcxOSwiaWF0IjoxNjEwNDAwODE5fQ.P6do" \
                           "VgTGDzPOVTIkUZH89FFaaVlC9zKg-94nDE5evGWxtgG_3jHViajXFEsDYlDN7SEZoX9IPN0cx2ddERl5HyBeMYpkS" \
                           "xDFxHCDMuiN1Boj9RpH45dZYzmJvCYGRdP_eOr2R7cHpZLWrS2K6DrHmHBlPoLQjRUUR1Y-ppMRm7M2FIl7pRARHj" \
                           "gA_cxoeMTovlNwoeSSNpYXYygPx0Z7YxSZI35NpWlpxUlfume9DE29yJ9zpjsndZ1H3eUQRxsQvXI0n3O8vvL91Gs" \
                           "xuBSlnR3Tt_jpdxtR2MaJnV7OVKQK9LUVhuNWoVP3e6RIIxzolLIFEI0BT-SkR5nVGYEgfQ"

        self.expired_cookie = "H4sIAAAAAAAA_4xVWZOjOBL-RxMIitrh0S4DhrZEI3QgvQFyN4eEVeUD0K_fcFXPzF4TsW-glL4rMyLPW963aT" \
                              "cUQ55RlwE0ZNdsxmH3lr1mk63ZWx79dt5y0Plsa01yk1X2mpmb16TJJkm8lCm7Ca6vp5ndxRZqeaTD6S13TY11" \
                              "t2WvHHjXbEKgC_CjpegquL49z5_EipdP4rHh60P60b0YorFNtW5n_KPh5QMa4SGCdZFCAF0yCLJz8jAF0GEjiH" \
                              "hBPnTQxGtRZdfMfOnIxsvQBPgi6lxDst9JDvqGL3fFw_Fk2KQ-78q-5fp6ZtFdcKCfb1ja951h7-Kzrqwy7F77" \
                              "Yd9y-tR4an3Qt2_PmnZP_ZnpPXXcu2L4_SF8fW199WiH8NGZ7un_T9zKj26CLz__DX9GQAx_YOxeT1v0ZwYnE2" \
                              "1yi5ysc1_W-f4UsOe3O7l4LcjOg2S3ZsMyqDT6kDx85vTUd1FHvHTu8jj5yLZpNLf-y70N8vnkR72q8QUOkZUJ" \
                              "fjQ-u5-c8OSB9YLnBrppQQRuaBShTMsV8ThEZAeEwVoYup5c7EOy84pD6UMnAjTGw49qGWicjMJnrgNfmJ-5H7" \
                              "HXHeH_8vMQNfOaQ_YQHI2yRo596n95QDL5grARHfZGOLGgA3TFgQJJVC-4WOUY--Kp0RcPSMQKD-ULOgiH3G6B" \
                              "zyz1vpJcbrJGl1-Y_28e23-8eyCS6-LwcxNuF0rOJuSXniT5JA8_X-TIBngQmyQUoMPu77Tkf2T8zEPW-htMmS" \
                              "i5_t5Qe2Jjghu6Hqo57xlILPF1fqZhTCeEmslbqbFN6YH3JpborJMXQvEFU0DLuq8YlTGZLYIT-CBkX1FPu4qD" \
                              "tAM2rIjM29hSwi2Dk77Dt1Cfqb0KrdIuzQ-f9bpcBFXpmVmKa4UxkB5n6tp5653NTJ4Ju_KpZ5_6TNjzxF6FWU" \
                              "LFdA35mjcmstRloUp0rXwNS8_eOOvWLs2zNpVDS6XHNcrRpK_URJLH9io4SL78Mslccq24fTQMbkWNJQvUzCZ1" \
                              "aVJ0Y3Mif_m5ng368sN_LjLu385TZIujlNCsToz5pExuUS3HliNLYvTtbHBdfeX7kHFfKe9v7_vC5fzM8Y3NGl" \
                              "M3rV__KCyOPWLpLSATTpQHSFWjvKVi5dOnnxuZ9wN2-i7GzEPTspEZ9QxIIGMRdok9FEfdNDpeyaQYSvQrqy1i" \
                              "nr0QiovGi6gIbIGBvRBjZwj0K6m1ZDEFFddBw_GxqPOGOWYEU7ihka1MNFVJHojAPtQn31_8imeuMiGEo77IpK" \
                              "9aD9ii3jdY574gOVd0feVk3-BEXjhT31oKXptAFSwGITFWImaPtO4L5lFANN5Dpm_UrLJl7IUcbQInQEuy_46T" \
                              "eCO1ldAL3zH_7PcXH_jp6LwfGGBf8xuH77hWBU67hZjnfGqrfN3wtA-e_f6X-ko0RnB6zsNe8uQ5___F1xRVmB" \
                              "X1bq3GfdIcsahYtKqDbiCLbrwuA8JR39XsXQK7nGr8nR_Rh5xR3zhtaMLiE9n7jcvC1ltcBfTY-HlCKI4bBr12" \
                              "UrqL1VS6cmljFohZvaixTzhTRrDecBP2zGUer_VNjfTWppK1gf6GeZKRCRA29wHTSUgACoVGCFMM-YguImCrmP" \
                              "GENrAv6ARaf_1QQd403MLaRxvyl0AEKOUHfEK-5IhhiNK1oJPiqBZhOU7beVS6O6BYxuytSNmdeRYoP1pJjJxK" \
                              "mEAJejsnCqEax2cyuS4tF-ThjyphRUnDDU2RpboMKpqT9rjXPNEMJrmhji6N6Quk5aED6oMae6qC_ACnnlETlR" \
                              "3B7Jxegparhzz0QUH2HxW_oQ7YV2nslXpqKniY8rpPWRxeaSAWZNiVUTmwVL8iQ18qfqOMgulU6xvTSHdJv3Tp" \
                              "5x7U5-NuKMZfO4RMC3TdXztUy4fi6JLN3m9bM7kd_z6kBfz9vkvucxvu0x_BbU3TAIe4Wd0_spNBb6VP_xkAAP" \
                              "__xxoL6rQIAAA="

        self.secret = "kmDDgMLGThapDV1QnhWPJd0oARzjLa5Zy3bQ8WfOIYk="

    def test_encode_decode_vouch_cookie(self):
        # Decode CI Logon Identity Token to get IDP Claims
        code, id_token = JWTManager.decode(cookie=self.valid_token, verify=False, compression=False)
        self.assertEqual(code, ValidateCode.VALID)
        self.assertTrue(isinstance(id_token, dict))

        p_tokens = PTokens(id_token=self.valid_token, idp_claims=id_token)

        encoder = VouchEncoder(secret="super-secret")

        # Create Vouch Cookie with validity 5 seconds
        code, cookie = encoder.encode(custom_claims_type=[CustomClaimsType.PROFILE, CustomClaimsType.EMAIL,
                                                          CustomClaimsType.OPENID],
                                      p_tokens=p_tokens, validity_in_seconds=5)

        self.assertEqual(code, ValidateCode.VALID)
        self.assertTrue(isinstance(cookie, str))

        # Decode the vouch cookie
        code, decoded_cookie = JWTManager.decode(cookie=cookie, compression=True, secret="super-secret")

        # Cookie decode successfully
        self.assertEqual(code, ValidateCode.VALID)
        self.assertTrue(isinstance(decoded_cookie, dict))

        # Sleep for 10 seconds, cookie has expired and the decode should fail now
        time.sleep(10)

        # Decoding Vouch cookie fails after 10 seconds
        code, decoded_cookie = JWTManager.decode(cookie=cookie, compression=True, secret="super-secret")
        self.assertEqual(code, ValidateCode.INVALID)

    def test_encode_vouch_cookie_missing_id_token(self):
        # Decode CI Logon Identity Token to get IDP Claims
        code, id_token = JWTManager.decode(cookie=self.valid_token, verify=False, compression=False)
        self.assertEqual(code, ValidateCode.VALID)
        self.assertTrue(isinstance(id_token, dict))

        p_tokens = PTokens(id_token=None, idp_claims=id_token)

        encoder = VouchEncoder(secret="super-secret")

        # Create Vouch Cookie with validity 5 seconds
        code, cookie = encoder.encode(custom_claims_type=[CustomClaimsType.PROFILE, CustomClaimsType.EMAIL,
                                                          CustomClaimsType.OPENID],
                                      p_tokens=p_tokens, validity_in_seconds=5)

        self.assertEqual(code, ValidateCode.UNSPECIFIED_ID_TOKEN)
        self.assertIsNone(cookie)

    def test_encode_vouch_cookie_missing_id_claims(self):
        # Decode CI Logon Identity Token to get IDP Claims
        code, id_token = JWTManager.decode(cookie=self.valid_token, verify=False, compression=False)
        self.assertEqual(code, ValidateCode.VALID)
        self.assertTrue(isinstance(id_token, dict))

        p_tokens = PTokens(id_token=self.valid_token, idp_claims=None)

        encoder = VouchEncoder(secret="super-secret")

        # Create Vouch Cookie with validity 5 seconds
        code, cookie = encoder.encode(custom_claims_type=[CustomClaimsType.PROFILE, CustomClaimsType.EMAIL,
                                                          CustomClaimsType.OPENID],
                                      p_tokens=p_tokens, validity_in_seconds=5)

        self.assertEqual(code, ValidateCode.UNSPECIFIED_ID_TOKEN)
        self.assertIsNone(cookie)

    def test_decode_expired_cookie(self):
        # decode cookie generated by Vouch without verifying
        code, decoded_cookie = JWTManager.decode(cookie=self.expired_cookie, compression=True, secret=self.secret,
                                                 verify=False)
        self.assertEqual(code, ValidateCode.VALID)

        # decode cookie generated by Vouch with verification
        code, decoded_cookie = JWTManager.decode(cookie=self.expired_cookie, compression=True, secret=self.secret)
        self.assertEqual(code, ValidateCode.INVALID)